#!/usr/bin/bash

# --- Vars ---
SCREEN=/dev/tty0

# --- End Vars ---

# --- Helpers ---
clearscreen () {
  tput -T xterm clear > $SCREEN
}
setpos () {
  tput -T xterm cup $1 $2 > $SCREEN
}

writetext () {
  echo "$1" > $SCREEN
}

writetext2pos () {
  setpos $1 $2
  #tput -T xterm el > $SCREEN
  writetext "                                                         "
  setpos $1 $2
  writetext "$3"
}

qrunflash () {

  setpos 28 0
  tput -T xterm-256color el > $SCREEN
  setpos 28 0
  writetext "      Flash Main      "
  setpos 28 26
  writetext "      Flash Parts     "
  setpos 28 52
  writetext "      Flash Voice     "
  setpos 28 78
  writetext "         Exit         "


  while read line
  do
    if [[ $line =~ .+90.+84 ]]; then
      echo "Main"
      break
    fi
    if [[ $line =~ .+91.+84 ]]; then
      echo "Parts"
      break
    fi 
    if [[ $line =~ .+92.+84 ]]; then
      echo "Voice"
      break
    fi 
    if [[ $line =~ .+93.+84 ]]; then
      echo "Exit"
      break
    fi 
  done < <(candump can0,90:7FF,91:7FF,92:7FF,93:7FF)

  setpos 28 0
  tput -T xterm-256color el > $SCREEN

}

flashMain () {
  
  FWFILE=$ABSDIR/FPMainEnc4.bin
  FWID=3
  
  if [ -f "$FWFILE" ]; then
    writetext2pos 6 2 '--> Flash firmware of Main Board ...'
    setpos 10 2
    $ABSDIR/CANFlashLoader -flash $FWFILE $FWID > $SCREEN
    sleep 2
    resetFlashOutput
    writetext2pos 6 2 '--> Flash firmware of Voice Board ...Ok'
  else
    writetext2pos 6 2 "Firmware file for Main Board not found."
    writetext2pos 7 2 "$FWFILE"
    sleep 5
  fi
}

flashParts () {
  
  FWFILE=$ABSDIR/FPPartEnv3.bin
  FWID=2

  if [ -f "$FWFILE" ]; then
    writetext2pos 6 2 '--> Flash firmware of Parts Board ...'
    setpos 10 2
    $ABSDIR/CANFlashLoader -flash $FWFILE $FWID > $SCREEN
    sleep 2
    resetFlashOutput
    writetext2pos 6 2 '--> Flash firmware of Voice Board ...Ok'
  else
    writetext2pos 6 2 "Firmware file for Parts Board not found."
    writetext2pos 7 2 "$FWFILE"
    sleep 5
  fi
}

flashVoice () {
  
  FWFILE=$ABSDIR/FPVoiceSection.bin
  FWID=1

  if [ -f "$FWFILE" ]; then
    writetext2pos 6 2 '--> Flash firmware of Voice Board ...'
    setpos 10 2
    $ABSDIR/CANFlashLoader -flash $FWFILE $FWID > $SCREEN
    sleep 2
    resetFlashOutput
    writetext2pos 6 2 '--> Flash firmware of Voice Board ...Ok'
  else
    writetext2pos 6 2 "Firmware file for Voice Board not found."
    writetext2pos 7 2 "$FWFILE"
    sleep 5
  fi
}

resetFlashOutput () {
  writetext2pos 10 0 " "
  writetext2pos 11 0 " "
  writetext2pos 12 0 " "
  writetext2pos 13 0 " "
  writetext2pos 14 0 " "
  writetext2pos 15 0 " "
}

# --- End Helpers ---

ABSPATH=$(readlink -f $0)
ABSDIR=$(dirname $ABSPATH)

cd $ABSDIR
zcat bg.dat > /dev/fb0

writetext2pos 6 2 'MD900 Firmware Flasher'
sleep 2

while true; do

  RES=$(qrunflash)

  case $RES in
    Main)
      flashMain
      RES=""
      ;;
    Parts)
      flashParts
      RES=""
      ;;
    Voice)
      flashVoice
      RES=""
      ;;
    *)
      writetext2pos 6 2 'Exit MD900 Firmware Flasher ...'
      sleep 3
      RES=""
      break 2
      ;;
  esac
done


writetext2pos 6 2 'Firmware Flasher done.'
sleep 3

